/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2011 Cassidian, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 * 
 * /!\ At the time of the 1.2 release, this module is still experimental and need more in 
 * depth use and validation. Use it at your own risks ;-)
 */

package org.ow2.weblab.rdf;

import java.net.URI;
import java.util.LinkedList;
import java.util.List;

import org.ow2.weblab.core.annotator.Element;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class RDFReader extends CommonNamespaces {

	private RDFReader() {
		throw new UnsupportedOperationException("This class only contains " + "static methods; no need to instantiate it.");
	}

	/**
	 * Read the value of a property described in RDF
	 * 
	 * @param <T>
	 *            the type of the value
	 * @param annotation
	 *            the annotation on which the property is written
	 * @param subject
	 *            the subject of the property
	 * @param namespace
	 *            the namespace of the property
	 * @param property
	 *            the localname of the property
	 * @param language
	 *            the language attribute of the property
	 * @param clazz
	 *            the class of the type T
	 * @return A Value object typed with T that contains all values of the property
	 */
	public static <T> Value<T> readValueFromStatement(PieceOfKnowledge annotation, URI subject, URI namespace, String property, String language,
			Class<T> clazz, Element element) {
		Value<T> res = new Value<T>();
		Node n = (Node) annotation.getData();

		/**
		 * We retrieve only the first description node of the subject
		 * There could be other description node about this subject
		 * TODO: support several description node about the same subject
		 */
		Node description = findElementAbout(n, subject.toString(), element);
		if (description == null) {
			return null;
		}

		// Check of properties
		String sNamespace = namespace.toString();
		if (property.indexOf(":") != -1 && property.indexOf("/") != -1) {
			int l = property.lastIndexOf('/');
			int d = property.lastIndexOf('#');
			d = d == -1 ? l : d;
			sNamespace = property.substring(0, d + 1);
			property = property.substring(d + 1);
		}

		List<Node> nodes = findNodesWithProperty(description, sNamespace, property);
		if (nodes.isEmpty()) {
			return null;
		}

		if (language == null) {

			for (Node node : nodes) {
				String value = getResource(node);
				if (value == null) {
					value = node.getTextContent();
				}
				NamedNodeMap nnm = node.getAttributes();
				Node nlang = nnm.getNamedItem("xml:lang");
				String lang = null;
				if (nlang != null) {
					lang = nlang.getTextContent();
				}
				res.addItem(XMLParser.xmlParse(value, clazz), lang);
			}

		}

		for (Node node : nodes) {
			NamedNodeMap nnm = node.getAttributes();
			Node nlang = nnm.getNamedItem("xml:lang");
			if (nlang != null) {
				String lang = nlang.getTextContent();
				if (lang.length() > 1 && language != null && language.equalsIgnoreCase(lang)) {
					String value = getResource(node);
					if (value == null) {
						value = node.getTextContent();
					}
					return new Value<T>(XMLParser.xmlParse(value, clazz));
				}
			}
		}

		return res;
	}

	protected static String getResource(Node n) {
		NamedNodeMap nnm = n.getAttributes();
		Node r = nnm.getNamedItemNS(RDF_URI, "resource");
		if (r == null) {
			return null;
		}
		return r.getTextContent();
	}

	protected static List<Node> findNodesWithProperty(Node n, String namespace, String property) {
		List<Node> list = new LinkedList<Node>();

		if (namespace.equals(n.getNamespaceURI()) && property.equalsIgnoreCase(n.getLocalName())) {
			list.add(n);
		}

		NamedNodeMap nnm = n.getAttributes();
		if (nnm != null) {
			for (int i = 0; i < nnm.getLength(); i++) {
				list.addAll(findNodesWithProperty(nnm.item(i), namespace, property));
			}
		}

		NodeList nl = n.getChildNodes();
		for (int i = 0; i < nl.getLength(); i++) {
			list.addAll(findNodesWithProperty(nl.item(i), namespace, property));
		}
		return list;
	}

	protected static String getLanguage(Node node) {
		NamedNodeMap nnm = node.getAttributes();
		if (nnm != null) {
			Node nlang = nnm.getNamedItem("xml:lang");
			if (nlang != null) {
				return nlang.getTextContent();
			}
		}
		return null;
	}

	/**
	 * Breadth first search should be more efficient than depth first search
	 * 
	 * @param n
	 *            a node
	 * @param subject
	 *            the subject we search
	 * @return the rdf:description node about the subject or null if it does not exist
	 */
	protected static Node findElementAboutIn(Node n, String subject, Element element) {
		if (n == null) {
			return null;
		}
		NodeList nl = n.getChildNodes();
		if (nl == null) {
			return null;
		}
		for (int i = 0; i < nl.getLength(); i++) {
			if (isElementAbout(nl.item(i), subject, element)) {
				return nl.item(i);
			}
		}

		for (int i = 0; i < nl.getLength(); i++) {
			Node d = findElementAboutIn(nl.item(i), subject, element);
			if (d != null) {
				return d;
			}
		}
		return null;
	}

	protected static Node findElementAbout(Node n, String subject, Element element) {
		if (isElementAbout(n, subject, element)) {
			return n;
		}
		return findElementAboutIn(n, subject, element);
	}

	protected static boolean isElementAbout(Node n, String subject, Element element) {
		if (n == null) {
			return false;
		}
		NamedNodeMap nnm = n.getAttributes();
		if (nnm != null && element.equals(n)) {
			Node att = nnm.getNamedItemNS(RDF_URI, "about");
			if (att != null) {
				String t = att.getTextContent();
				if (t != null && t.equalsIgnoreCase(subject)) {
					return true;
				}
			}
		}
		return false;
	}

	protected static Node findRDF(Node dom) {
		NamedNodeMap nnm = dom.getAttributes();
		if (nnm != null && "RDF".equalsIgnoreCase(dom.getLocalName())) {
			return dom;
		}

		NodeList nl = dom.getChildNodes();
		for (int i = 0; i < nl.getLength(); i++) {
			Node d = findRDF(nl.item(i));
			if (d != null) {
				return d;
			}
		}
		return null;
	}

	protected static void print(Node n, String space) {
		System.out.println(space + n);
		NodeList nl = n.getChildNodes();
		for (int i = 0; i < nl.getLength(); i++) {
			print(nl.item(i), space + "\t");
		}
	}
}
