/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2011 Cassidian, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 * 
 * /!\ At the time of the 1.2 release, this module is still experimental and need more in 
 * depth use and validation. Use it at your own risks ;-)
 */

package org.ow2.weblab.core.annotator;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Date;

import junit.framework.TestCase;

import org.geonames.www.ontology.WGeonamesAnnotator;
import org.ow2.weblab.core.annotator.AbstractAnnotator.Operator;
import org.ow2.weblab.core.extended.exception.WebLabCheckedException;
import org.ow2.weblab.core.extended.factory.ResourceFactory;
import org.ow2.weblab.core.extended.jaxb.WebLabMarshaller;
import org.ow2.weblab.core.model.Document;
import org.ow2.weblab.core.model.LinearSegment;
import org.ow2.weblab.core.model.Resource;
import org.ow2.weblab.core.model.Text;
import org.ow2.weblab.core.model.processing.WProcessingAnnotator;
import org.purl.dc.elements.DublinCoreAnnotator;
import org.w3.owl.OWLAnnotator;

public class AnnotatorTest extends TestCase {

	public void testDublinCoreAnnotator() throws URISyntaxException {
		Document document = ResourceFactory.createResource("idfact", "myDoc", Document.class);

		DublinCoreAnnotator dca = new DublinCoreAnnotator(document);
		BaseAnnotator rra = new BaseAnnotator(document);

		// Set several title
		dca.writeTitle("My New Document");
		dca.startSpecifyLanguage("fr");
		dca.writeTitle("Mon Nouveau Document");
		dca.endSpecifyLanguage();


		// Retrieve all title
		assertEquals("My New Document", dca.readTitle().toString());


		URI docType = new URI("weblab://weblab/type#document");
		// Set the type of the document as a rdf:resource
		rra.writeType(docType);
		assertEquals(docType, rra.readType().firstTypedValue());

		// get french title
		dca.startSpecifyLanguage("fr");
		assertEquals("Mon Nouveau Document", dca.readTitle().toString());
		dca.endSpecifyLanguage();

		Date d = new Date();
		dca.writeDate(d);
		Date d2 = dca.readDate().firstTypedValue();
		assertEquals(d.toString(), d2.toString());



	}

	public void testWProcessingAnnotator() throws URISyntaxException {
		Text text = ResourceFactory.createResource("idfact", "myDoc", Text.class);
		text.setContent("London.");

		// We can use several Annotator on a Resource at the same time
		WProcessingAnnotator wpa = new WProcessingAnnotator(text);
		WGeonamesAnnotator geoa = new WGeonamesAnnotator(text);

		wpa.writeProducedBy(URI.create("myservice"));
		
		// We create URI
		URI segmentURI = new URI("weblab://idfact/mySegment");
		URI londonGeonamesURI = new URI("http://sws.geonames.org/2643743/");

		// Create a geographic entity in the document from an external ontology
		geoa.setElement("gn", "http://www.geonames.org/ontology#", "Feature");
		geoa.startInnerAnnotatorOn(londonGeonamesURI);

		// We set RDFS labels
		// We set the language support in Official Aramaic ISO format
		geoa.startSpecifyLanguage("arc");
		// We write label in the Official Aramaic language
		geoa.writeLabel("ܠܘܢܕܘܢ ");


		// Wet the language support in French ISO format
		geoa.startSpecifyLanguage("fr");
		// We write label in the French language
		geoa.writeLabel("La ville de Londres");

		// We set the overwrite mode to reset the french label
		geoa.setAppendMode(false);

		// We overwrite a label
		geoa.writeLabel("Londres");
		assertEquals("Londres", geoa.readLabel().toString());

		// We set back the append mode of the annotator
		geoa.setAppendMode(true);

		// We remove language specification
		geoa.endSpecifyLanguage();
		// we write a label without language attribute
		geoa.writeLabel("London");

		// We set geonames specific latitude/longitude properties
		geoa.writeWgs84_pos_lat(51.50051);
		geoa.writeWgs84_pos_long(-0.12883);

		// Create add segment on the Text
		LinearSegment segment = new LinearSegment();
		segment.setStart(0);
		segment.setEnd(6);
		segment.setUri(segmentURI.toString());
		text.getSegment().add(segment);

		// Create a semantic link between the segment and the geographical entity
		wpa.startInnerAnnotatorOn(segmentURI);
		wpa.writeRefersTo(londonGeonamesURI);


		OWLAnnotator oa = new OWLAnnotator(text);
		oa.writeSameAs(URI.create("test"));

		print(text);

		wpa.writeCandidate(true);
		assertNull(geoa.readPopulation());

		assertEquals(51.50051, geoa.readWgs84_pos_lat().firstTypedValue());
		assertEquals(-0.12883, geoa.readWgs84_pos_long().firstTypedValue());

		// assertEquals(true, wpa.readCandidate().firstTypedValue().booleanValue());
		// assertNull(wpa.readOriginalFileName());

		/**
		 * Test with operators
		 */
		assertNull(wpa.applyOperator(Operator.SET, "none", URI.create("none:uri"), "none;prop", String.class, "testValue"));

	}
	
	public void testElements(){
		
		Resource resource = ResourceFactory.createResource("id", "res", Resource.class);
		
		BaseAnnotator ba = new BaseAnnotator(resource);
		ba.setDefaultElement();
		Element element = new Element("prefix", "uri", "localName");
		element.setLocalName("localName");
		element.setUri("uri");
		element.setPrefix("prefix");
		ba.setElement(element);
		
		assertEquals("urilocalName", element.toURI().toString());
		
	}

	public void print(Resource r) {
		try {
			new WebLabMarshaller().marshalResource(r, System.out);
		} catch (WebLabCheckedException e) {
			e.printStackTrace();
		}
	}
}
