/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2011 Cassidian, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 * 
 * /!\ At the time of the 1.2 release, this module is still experimental and need more in 
 * depth use and validation. Use it at your own risks ;-)
 */

package org.ow2.weblab.core.annotator;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

import junit.framework.TestCase;

import org.geonames.www.ontology.WGeonamesAnnotator;
import org.ow2.weblab.core.extended.exception.WebLabCheckedException;
import org.ow2.weblab.core.extended.factory.ResourceFactory;
import org.ow2.weblab.core.extended.jaxb.WebLabMarshaller;
import org.ow2.weblab.core.model.Document;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.ow2.weblab.core.model.Resource;
import org.ow2.weblab.core.model.WModelAnnotator;
import org.ow2.weblab.core.model.processing.WProcessingAnnotator;
import org.ow2.weblab.core.model.retrieval.WRetrievalAnnotator;
import org.ow2.weblab.rdf.Value;
import org.purl.dc.elements.DublinCoreAnnotator;
import org.purl.dc.terms.DCTermsAnnotator;
import org.w3.owl.OWLAnnotator;

public class TestAnnotators extends TestCase {

	public void testAnnotators() {
		Document doc = ResourceFactory.createResource("ref", "test", Document.class);
		BaseAnnotator[] bas = new BaseAnnotator[] { 
				new WProcessingAnnotator(doc), 
				new WRetrievalAnnotator(doc), 
				new WGeonamesAnnotator(doc),
				new DublinCoreAnnotator(doc), 
				new DCTermsAnnotator(doc), 
				new WModelAnnotator(doc),
				new OWLAnnotator(doc)};
		URI testuri = URI.create("test:test");
		WProcessingAnnotator wpa = new WProcessingAnnotator(testuri, new PieceOfKnowledge());
		new WRetrievalAnnotator(testuri, new PieceOfKnowledge());
		new WGeonamesAnnotator(testuri, new PieceOfKnowledge());
		new DublinCoreAnnotator(testuri, new PieceOfKnowledge());
		new DCTermsAnnotator(testuri, new PieceOfKnowledge());
		new WModelAnnotator(testuri, new PieceOfKnowledge());
		new OWLAnnotator(testuri, new PieceOfKnowledge());

		for (BaseAnnotator base : bas) {
			browseAnnotator(base);
		}

		assertTrue(wpa.isAppendMode());
		wpa.startInnerAnnotatorOn(URI.create("none:none"));
		wpa.endInnerAnnotator();
		wpa.resource = doc;
		wpa.endInnerAnnotator();
		assertFalse(wpa.isInnerAnnotatorStarted);
		new WProcessingAnnotator(URI.create("none:er"), null);
		doc.setUri("not an uri");
		new WProcessingAnnotator(doc);

	}

	public void testDCAnnotatorOnOneDoc() throws WebLabCheckedException {
		Document doc = ResourceFactory.createResource("test", "doc", Document.class);
		DublinCoreAnnotator annot = new DublinCoreAnnotator(doc);
		annot.writeLanguage("eng");
		new WebLabMarshaller().marshalResource(doc, new File("./target/test.xml"));
		System.out.println(annot.readLanguage());
	}

	private void browseAnnotator(BaseAnnotator base) {
		List<Method> write = new LinkedList<Method>();
		List<Method> read = new LinkedList<Method>();
		for (Method f : base.getClass().getDeclaredMethods()) {
			if (f.getName().startsWith("read")) {
				read.add(f);
			} else if (f.getName().startsWith("write")) {
				write.add(f);
			}
		}

		for (Method wm : write) {
			try {
				wm.invoke(base, getParameter(wm.getParameterTypes()[0]));
			} catch (Exception e) {
				e.printStackTrace();
				fail();
			}
		}

		for (Method wm : read) {
			try {
				Value<?> o = (Value<?>) wm.invoke(base);

				assertNotNull(o);
				assertTrue(o.getValues().size() > 0);
				Object z = o.getValues().getFirst();
				Object res = getParameter(z.getClass());
				assertEquals(res, z);

			} catch (Exception e) {
				e.printStackTrace();
				fail();
			}
		}
	}

	private Object getParameter(Class<?> clazz) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ", Locale.ENGLISH);
		Date now = null;
		try {
			now = sdf.parse("2010-09-14T14:17:49+0200");
		} catch (ParseException e1) {
			e1.printStackTrace();
		}
		String test = "test";
		URI uri = null;

		try {
			uri = new URI("test:test");
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}

		boolean b = true;
		int t = 12;
		double d = 0.241;
		long l = 1235435431453453152l;
		if (clazz.isInstance(now)) {
			return now;
		} else if (clazz.isInstance(uri)) {
			return uri;
		} else if (clazz.isInstance(test)) {
			return test;
		} else if (clazz.isInstance(l)) {
			return l;
		} else if (clazz.isInstance(t)) {
			return t;
		} else if (clazz.isInstance(d)) {
			return d;
		} else if (clazz.isInstance(b)) {
			return b;
		}
		return null;
	}

	public void testT() {

		try {
			Resource res = new WebLabMarshaller().unmarshal(new File("./target/test.xml"), Resource.class);
			DublinCoreAnnotator annotator = new DublinCoreAnnotator(res);
			assertEquals("eng", annotator.readLanguage().firstTypedValue());

		} catch (WebLabCheckedException e) {
			e.printStackTrace();
			fail();
		}

	}

}
