/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2011 CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */

package org.ow2.weblab.core.extended.factory;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.ow2.weblab.core.extended.exception.WebLabUncheckedException;
import org.ow2.weblab.core.extended.uri.WebLabRI;
import org.ow2.weblab.core.model.Annotation;
import org.ow2.weblab.core.model.ComposedQuery;
import org.ow2.weblab.core.model.ComposedResource;
import org.ow2.weblab.core.model.Document;
import org.ow2.weblab.core.model.LowLevelDescriptor;
import org.ow2.weblab.core.model.MediaUnit;
import org.ow2.weblab.core.model.Query;
import org.ow2.weblab.core.model.Resource;
import org.ow2.weblab.core.model.ResultSet;
import org.ow2.weblab.core.model.Segment;
import org.ow2.weblab.core.model.SimilarityQuery;
import org.ow2.weblab.core.model.TrackSegment;

/**
 * 
 * This class provides methods to generate URI that do not already exist in a Resource.
 * 
 * Resource creation factories mostly use only two methods:
 * <ul>
 * <li>createUniqueURIFrom(resource, isChild, isAnnotation) {</li>
 * <li>createSegmentURIFor(segmentContainer)</li>
 * </ul> 
 * 
 * @author Cassidian WebLab Team
 * @since 1.2
 * @date 2010-11-02
 */
abstract class AbstractFactory {

	/**
	 * This data structure is not well adapted to manage WebLab Resource
	 */
	final protected static Map<String, Set<String>> innerMap = Collections.synchronizedMap(new LinkedHashMap<String, Set<String>>());

	protected synchronized static void cleanMapping(final String uri) {
		final Iterator<Entry<String, Set<String>>> it = AbstractFactory.innerMap.entrySet().iterator();
		while (it.hasNext()) {
			final Entry<String, Set<java.lang.String>> entry = it.next();
			if (entry.getValue().contains(uri)) {
				it.remove();
			}
		}
	}

	protected synchronized static Set<String> getUris(final String key) {
		if (!AbstractFactory.innerMap.containsKey(key)) {
			for (final Set<String> test : AbstractFactory.innerMap.values()) {
				if (test.contains(key)) {
					return test;
				}
			}
			return new HashSet<String>();
		}
		return AbstractFactory.innerMap.get(key);
	}

	protected synchronized static void setUris(final String key, final Set<String> uris) {
		while (AbstractFactory.innerMap.size() > 1000) {
			final String fkey = AbstractFactory.innerMap.keySet().iterator().next();
			AbstractFactory.innerMap.remove(fkey);
		}
		AbstractFactory.innerMap.put(key, uris);
	}

	/**
	 * List uris in the resource
	 * 
	 * @param resource
	 *            a resource
	 * @return a set of uri in this resource
	 */
	protected synchronized static Set<String> listUri(final Resource resource) {
		return AbstractFactory.listUri(resource, false);
	}

	private static Set<String> listUri(final String uri) {
		final Set<String> buffered_uris = AbstractFactory.getUris(uri);
		return buffered_uris;
	}

	/**
	 * Lists all URI in a segment and its subsegments.
	 * @param segment a segment
	 * @returna list of all URI in this segment the last time this segment was processed
	 */
	protected synchronized static Set<String> listUri(final Segment segment) {
		final Set<String> buffered_uris = AbstractFactory.listUri(segment.getUri());
		if (!buffered_uris.isEmpty()) {
			return buffered_uris;
		}
		final Set<String> set = Collections.synchronizedSet(new HashSet<String>());
		set.add(segment.getUri());
		
		if (segment instanceof TrackSegment){
			final TrackSegment ts = (TrackSegment)segment;
			for (final Segment inner : ts.getSegment()) {
				set.addAll(listUri(inner));
			}
		}
		
		return set;
	}
	
	/**
	 * Lists all URI in a resource and its subresources.
	 * @warning this function does not support cyclic resources (a resource that contains itself) if force is true 
	 * 
	 * @param resource a resource
	 * @param force if false URI in this resource will not be listed if the resource have already been processed, else they will not matter what.   
	 * @return a list of all URI in this resource the last time this resource was processed
	 */
	protected synchronized static Set<String> listUri(final Resource resource, final boolean force) {
		final Set<String> buffered_uris = AbstractFactory.listUri(resource.getUri());
		if (!force && !buffered_uris.isEmpty()) {
			return buffered_uris;
		}
		final Set<String> set = Collections.synchronizedSet(new HashSet<String>());
		set.add(resource.getUri());
		if (resource instanceof Document) {
			final Document doc = (Document) resource;
			if (doc.isSetMediaUnit()) {
				for (final MediaUnit inner : doc.getMediaUnit()) {
					set.addAll(AbstractFactory.listUri(inner));
				}
			}
			if (doc.isSetSegment()) {
				for (final Segment s : doc.getSegment()) {
					set.addAll(AbstractFactory.listUri(s));
				}
			}
		} else if (resource instanceof ComposedResource) {
			final ComposedResource cr = (ComposedResource) resource;
			for (final Resource inner : cr.getResource()) {
				set.addAll(AbstractFactory.listUri(inner));
			}
		} else if (resource instanceof MediaUnit) {
			final MediaUnit mu = (MediaUnit) resource;
			if (mu.isSetSegment()) {
				for (final Segment s : mu.getSegment()) {
					set.addAll(AbstractFactory.listUri(s));
				}
			}
		} else if (resource instanceof ResultSet) {
			final ResultSet rs = (ResultSet)resource;
			if (rs.isSetPok()){
				set.addAll(AbstractFactory.listUri(rs.getPok()));
			}
			
		} else if (resource instanceof ComposedQuery) {
			final ComposedQuery cq = (ComposedQuery)resource;
			for(final Query query:cq.getQuery()){
				set.addAll(AbstractFactory.listUri(query));
			}
		} else if (resource instanceof SimilarityQuery){
			final SimilarityQuery sq = (SimilarityQuery)resource;
			for(final Resource subresource:sq.getResource()){
				set.addAll(AbstractFactory.listUri(subresource));
			}
		}

		if (resource.isSetAnnotation()) {
			for (final Annotation a : resource.getAnnotation()) {
				set.addAll(AbstractFactory.listUri(a));
			}
		}
		if (resource.isSetDescriptor()) {
			for (final LowLevelDescriptor lld : resource.getDescriptor()) {
				set.addAll(AbstractFactory.listUri(lld));
			}
		}
		AbstractFactory.setUris(resource.getUri(), set);

		return set;
	}

	protected synchronized static String createSegmentURIFor(final Object segmentContainer) {
		if (segmentContainer instanceof Segment){
			final Segment segment = (Segment)segmentContainer; 
			return AbstractFactory.createUniqueURIin(AbstractFactory.listUri(segment), true, false, segment.getUri());
		}else if (segmentContainer instanceof Resource){
			final Resource resource = (Resource)segmentContainer;
			return AbstractFactory.createUniqueURIin(AbstractFactory.listUri(resource), true, false, resource.getUri());
		}else{
			throw new WebLabUncheckedException("Object of type "+segmentContainer.getClass()+" can not contain Segment.");
		}
		
	}
	
	/**
	 * Create a URI checking it does not already exist in the resource.
	 *  
	 * @param resource a top container resource 
	 * @param isChild if true, the uri will describe a child or subchild of this resource
	 * @param isAnnotation if true, the uri will describe a annotation on this resource
	 * @return an URI unique among all URI in the resource and its sub resources
	 */
	protected synchronized static String createUniqueURIFrom(final Resource resource, final boolean isChild, final boolean isAnnotation) {
		return AbstractFactory.createUniqueURIin(AbstractFactory.listUri(resource), isChild, isAnnotation, resource.getUri());
	}

	protected synchronized static String createUniqueURIin(final Set<String> uris, final boolean isChild, final boolean isAnnotation, final String parentUri) {
		final String ref = "webLabFactory";
		final String res = "auto-parent";
		String template = "weblab://" + ref + "/" + res;
		if (isAnnotation) {
			if (parentUri.indexOf("#") != -1) {
				template = parentUri + "-a";
			} else {
				template = parentUri + "#a";
			}
		}
		if (isChild) {
			if (parentUri.indexOf("#") != -1) {
				template = parentUri + "-";
			} else {
				template = parentUri + "#";
			}
		}
		int i = uris.size() - 1;

		String uri = null;
		while ((uri == null) || uris.contains(uri)) {
			if (i + 1 % 1000 == 0) {
				template += i + "_";
				i = 0;
			}
			uri = template + (i++);
		}
		
		uris.add(uri);
		
		return new WebLabRI(uri).toString();
	}

	protected synchronized static void removeChild(final MediaUnit mu, final Resource parent, final Resource root) {
		final Set<String> uris = AbstractFactory.listUri(root);
		if (parent instanceof ComposedResource) {
			final ComposedResource cr = (ComposedResource) parent;
			cr.getResource().remove(mu);

			uris.remove(mu.getUri());
		}
	}

	protected synchronized static void addAnnotation(final Annotation annotation, final Resource resource, final Resource root) {
		resource.getAnnotation().add(annotation);
		AbstractFactory.listUri(root).add(annotation.getUri());
	}

	protected synchronized static void addSegment(final Segment segment, final MediaUnit parent, final Resource root) {
		parent.getSegment().add(segment);
		AbstractFactory.listUri(root).add(segment.getUri());
	}

	protected synchronized static void addDescriptor(final LowLevelDescriptor descriptor, final Resource parent, final Resource root) {
		parent.getDescriptor().add(descriptor);
		AbstractFactory.listUri(root).add(descriptor.getUri());
	}

	/**
	 * @deprecated since model 1.2, no need to separates root and parent.
	 *             replaced by addChild(final MediaUnit mu, final Document parent)
	 */
	@Deprecated
	protected synchronized static void addChild(final MediaUnit mu, final Resource parent, final Resource root) {
		final Set<String> uris = AbstractFactory.listUri(root);
		if (parent instanceof ComposedResource) {
			final ComposedResource cr = (ComposedResource) parent;
			cr.getResource().add(mu);
			uris.add(mu.getUri());
		} else if (parent instanceof Document) {
			final Document doc = (Document) parent;
			doc.getMediaUnit().add(mu);
			uris.add(mu.getUri());
		} else {
			throw new WebLabUncheckedException("Unable to create a sub media unit on a: " + parent.getClass() + " needed " + Document.class + " or "
					+ ComposedResource.class);
		}

	}

	protected synchronized static void addChild(final MediaUnit mu, final Document parent) {
		final Set<String> uris = AbstractFactory.listUri(parent);
		parent.getMediaUnit().add(mu);
		uris.add(mu.getUri());

	}

	protected synchronized static Resource findParent(final MediaUnit mu, final Resource resource) {
		if (resource instanceof ComposedResource) {
			final ComposedResource cr = (ComposedResource) resource;
			if (cr.getResource().contains(mu)) {
				return resource;
			}
			for (final Resource inner : cr.getResource()) {
				final Resource r = AbstractFactory.findParent(mu, inner);
				if (r != null) {
					return r;
				}
			}
		} else if (resource instanceof Document) {
			final Document doc = (Document) resource;
			if (doc.isSetMediaUnit() && doc.getMediaUnit().contains(mu)) {
				return resource;
			}
		}
		return null;
	}

	/**
	 * @param res
	 *            The <code>Resource</code> to be used to create <code>WebLabRI</code>
	 * @return The <code>String</code> to be used as <code>URI</code> by the <code>MediaUnit</code> created
	 */
	protected synchronized final static String getUniqueWebLabRIFrom(final Resource res, final boolean isChild, final boolean isAnnotation) {
		return AbstractFactory.createUniqueURIFrom(res, isChild, isAnnotation);
	}
}
