/**
* WEBLAB: Service oriented integration platform for media mining and intelligence applications
*  
* Copyright (C) 2004 - 2011 Cassidian, an EADS company
* 
* This library is free software; you can redistribute it and/or modify it under the terms of
* the GNU Lesser General Public License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License along with this
* library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
* Floor, Boston, MA 02110-1301 USA
*/

package org.ow2.weblab.core.helper.impl;

import javax.sound.sampled.Line;

import org.apache.commons.logging.LogFactory;
import org.ow2.weblab.core.extended.ontologies.RDF;
import org.ow2.weblab.core.extended.ontologies.WebLab;
import org.ow2.weblab.core.extended.util.PoKUtil;
import org.ow2.weblab.core.model.Annotation;
import org.ow2.weblab.core.model.Audio;
import org.ow2.weblab.core.model.ComposedQuery;
import org.ow2.weblab.core.model.ComposedResource;
import org.ow2.weblab.core.model.Document;
import org.ow2.weblab.core.model.Image;
import org.ow2.weblab.core.model.LinearSegment;
import org.ow2.weblab.core.model.LowLevelDescriptor;
import org.ow2.weblab.core.model.MediaUnit;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.ow2.weblab.core.model.Query;
import org.ow2.weblab.core.model.Resource;
import org.ow2.weblab.core.model.ResultSet;
import org.ow2.weblab.core.model.Segment;
import org.ow2.weblab.core.model.SimilarityQuery;
import org.ow2.weblab.core.model.SpatialSegment;
import org.ow2.weblab.core.model.StringQuery;
import org.ow2.weblab.core.model.TemporalSegment;
import org.ow2.weblab.core.model.Text;
import org.ow2.weblab.core.model.Video;


/**
 * This is an implementation of the <code>ResourceHelperExtended</code>. It loads every <code>Annotations</code> that are on this object and in the inner
 * <code>Resources</code>. It adds the structure statements in the generated inner model. If the loaded <code>Resource</code> is an <code>Annotation</code>, its
 * content won't be loaded.
 * 
 * @author EADS WebLab Team
 * @date 2009-02-10
 * @todo Prevent the direct instantiation of this class.
 */
public class JenaResourceStructureHelperExtended extends JenaResourceHelperExtended {

	/**
	 * Method recursively called to add every resources in the model and in the map.
	 * 
	 * @param res
	 *            The resource to add.
	 */
	@Override
	protected void addResource(final Resource res) {
		this.resourcesMap.put(res.getUri(), res);
		for (final Annotation annot : res.getAnnotation()) {
			this.bejh.createResStat(res.getUri(), WebLab.HAS_ANNOTATION, annot.getUri());
			this.bejh.createResStat(annot.getUri(), WebLab.IN_RESOURCE_ANNOTATION, res.getUri());
			this.addResource(annot);
		}
		for (final LowLevelDescriptor lld : res.getDescriptor()) {
			this.bejh.createResStat(res.getUri(), WebLab.HAS_LOW_LEVEL_DESCRIPTOR, lld.getUri());
			this.bejh.createResStat(lld.getUri(), WebLab.IN_RESOURCE_LLD, res.getUri());
			this.addResource(lld);
		}

		if (res instanceof MediaUnit) {
			this.addMediaUnit((MediaUnit) res);
		} else if (res instanceof Annotation) {
			this.addPieceOfKnowledge((PieceOfKnowledge) res);
		} else if (res instanceof Query) {
			this.addQuery((Query) res);
		} else if (res instanceof ResultSet) {
			this.addResultSet((ResultSet) res);
		} else if (res instanceof ComposedResource) {
			this.addComposedResource((ComposedResource) res);
		} else if (res instanceof LowLevelDescriptor) {
			this.bejh.createResStat(res.getUri(), RDF.TYPE, WebLab.getUriFromClass(LowLevelDescriptor.class));
		} else {
			if (!res.getClass().equals(Resource.class)) {
				LogFactory.getLog(this.getClass()).warn("Resource type unknown: " + res.getClass().getCanonicalName());
			}
			this.bejh.createResStat(res.getUri(), RDF.TYPE, WebLab.getUriFromClass(Resource.class));
		}

	}

	/**
	 * @param q
	 *            The query to add.
	 */
	protected void addQuery(final Query q) {
		if (q instanceof StringQuery) {
			this.bejh.createResStat(q.getUri(), RDF.TYPE, WebLab.getUriFromClass(StringQuery.class));
		} else if (q instanceof SimilarityQuery) {
			this.addSimilarityQuery((SimilarityQuery) q);
		} else if (q instanceof ComposedQuery) {
			this.addComposedQuery((ComposedQuery) q);
		} else {
			LogFactory.getLog(this.getClass()).warn("Unknown type of Query: " + q.getClass().getCanonicalName());
			this.bejh.createResStat(q.getUri(), RDF.TYPE, WebLab.getUriFromClass(Query.class));
		}
	}

	protected void addSimilarityQuery(final SimilarityQuery sq) {
		this.bejh.createResStat(sq.getUri(), RDF.TYPE, WebLab.getUriFromClass(SimilarityQuery.class));
		for (final Resource r : sq.getResource()) {
			this.bejh.createResStat(sq.getUri(), WebLab.HAS_RESOURCE_SIM_QUERY, r.getUri());
			this.bejh.createResStat(r.getUri(), WebLab.IN_SIMILARITY_QUERY, sq.getUri());
			this.addResource(r);
		}
	}

	protected void addComposedQuery(final ComposedQuery cq) {
		this.bejh.createResStat(cq.getUri(), RDF.TYPE, WebLab.getUriFromClass(ComposedQuery.class));
		for (final Query q : cq.getQuery()) {
			this.bejh.createResStat(cq.getUri(), WebLab.HAS_QUERY_IN_COMPOSED, q.getUri());
			this.bejh.createResStat(q.getUri(), WebLab.IN_COMPOSED_QUERY, cq.getUri());
			this.addResource(q);
		}
	}

	protected void addResultSet(final ResultSet rs) {
		this.bejh.createResStat(rs.getUri(), RDF.TYPE, WebLab.getUriFromClass(ResultSet.class));
		for (final Resource r : rs.getResource()) {
			this.bejh.createResStat(rs.getUri(), WebLab.HAS_RESOURCE_RESULT, r.getUri());
			this.bejh.createResStat(r.getUri(), WebLab.IN_RESULT_SET_RESOURCE, rs.getUri());
			this.addResource(r);
		}
		final PieceOfKnowledge p = rs.getPok();
		if (p != null) {
			this.bejh.createResStat(rs.getUri(), WebLab.HAS_POK_RESULT, p.getUri());
			this.bejh.createResStat(p.getUri(), WebLab.IN_RESULT_SET_POK, rs.getUri());
			this.addResource(p);
		}
	}

	protected void addComposedResource(final ComposedResource rc) {
		for (final Resource r : rc.getResource()) {
			this.bejh.createResStat(rc.getUri(), WebLab.HAS_RESOURCE, r.getUri());
			this.bejh.createResStat(r.getUri(), WebLab.IN_COMPOSED_RESOURCE, rc.getUri());
			this.addResource(r);
		}
		this.bejh.createResStat(rc.getUri(), RDF.TYPE, WebLab.getUriFromClass(ComposedResource.class));
	}


	@Override
	protected void addPieceOfKnowledge(final PieceOfKnowledge p) {
		final String rdf = PoKUtil.getPoKData(p);
		if (!rdf.trim().equals("")) {
			this.loadFromRdfXml(rdf, true);
		}
		this.bejh.createResStat(p.getUri(), RDF.TYPE, WebLab.getUriFromClass(Annotation.class));
	}

	/**
	 * @param seg
	 *            The segment to add.
	 */
	protected void addSegment(Segment seg) {
		if (seg instanceof LinearSegment) {
			this.bejh.createResStat(seg.getUri(), RDF.TYPE, "http://weblab-project.org/core/model/class/LinearSegment");
		} else if (seg instanceof TemporalSegment) {
			this.bejh.createResStat(seg.getUri(), RDF.TYPE, "http://weblab-project.org/core/model/class/TemporalSegment");
		} else if (seg instanceof SpatialSegment) {
			this.bejh.createResStat(seg.getUri(), RDF.TYPE, "http://weblab-project.org/core/model/class/SpatialSegment");
		} else {
			LogFactory.getLog(this.getClass()).warn("Unknown type of Segment: " + seg.getClass().getCanonicalName());
			this.bejh.createResStat(seg.getUri(), RDF.TYPE, "http://weblab-project.org/core/model/class/Segment");
		}
	}

	/**
	 * @param mu
	 *            The mediaUnit to add.
	 */
	protected void addMediaUnit(final MediaUnit mu) {
		for (final Segment seg : (mu).getSegment()) {
			this.bejh.createResStat(mu.getUri(), WebLab.HAS_SEGMENT, seg.getUri());
			this.bejh.createResStat(seg.getUri(), WebLab.IN_MEDIA_UNIT, mu.getUri());
			this.addSegment(seg);
		}

		if (mu instanceof Document) {
			this.addDocument((Document) mu);
		} else if (mu instanceof Text) {
			this.bejh.createResStat(mu.getUri(), RDF.TYPE, WebLab.getUriFromClass(Text.class));
		} else if (mu instanceof Image) {
			this.bejh.createResStat(mu.getUri(), RDF.TYPE, WebLab.getUriFromClass(Image.class));
		} else if (mu instanceof Audio) {
			this.bejh.createResStat(mu.getUri(), RDF.TYPE, WebLab.getUriFromClass(Audio.class));
		} else if (mu instanceof Video) {
			this.bejh.createResStat(mu.getUri(), RDF.TYPE, WebLab.getUriFromClass(Video.class));
		} else {
			LogFactory.getLog(this.getClass()).warn("Unknown type of MediaUnit: " + mu.getClass().getCanonicalName());
			this.bejh.createResStat(mu.getUri(), RDF.TYPE, WebLab.getUriFromClass(MediaUnit.class));
		}
	}

	/**
	 * @param cu
	 */
	protected void addDocument(final Document cu) {
		for (final MediaUnit mu : cu.getMediaUnit()) {
			this.bejh.createResStat(cu.getUri(), WebLab.HAS_MEDIA_UNIT, mu.getUri());
			this.bejh.createResStat(mu.getUri(), WebLab.IN_COMPOSED_UNIT, cu.getUri());
			this.addResource(mu);
		}
		
		this.bejh.createResStat(cu.getUri(), RDF.TYPE, WebLab.getUriFromClass(Document.class));
		 
	}

}
