/**
* WEBLAB: Service oriented integration platform for media mining and intelligence applications
*  
* Copyright (C) 2004 - 2011 Cassidian, an EADS company
* 
* This library is free software; you can redistribute it and/or modify it under the terms of
* the GNU Lesser General Public License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License along with this
* library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
* Floor, Boston, MA 02110-1301 USA
*/

package org.ow2.weblab.core.helper.impl;

import java.io.OutputStream;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.ow2.weblab.core.extended.factory.ResourceFactory;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.ow2.weblab.core.model.Resource;

import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.shared.ClosedException;


/**
 * 
 * @author emilienbondu
 *
 * A semantic resource implementation based on Jena ARQ.
 * A semantic resource can process SPARQL queries on the semantic description of the
 * WebLab resource and, optionally, reason on external ontologies.
 */
public class SemanticResource {

	//attributes
	protected Model model; 
	protected Resource resource;
	private FastResourceToJenaModel loader;
	private Set<String> prefix;
	
	/**
	 * To construct a semantic resource from a WebLab Resource.
	 * Use a JenaDefaultModel as model, and fill it with :
	 * - Statements from the WebLab Resource.
	 * - Statements about the WebLab structure of the resource are also added in the model.
	 * 
	 * Set rdf and rdfs prefixes for SPARQL queries to :
	 * - PREFIX rdf:<http://www.w3.org/1999/02/22-rdf-syntax-ns#>
	 * - PREFIX rdfs:<http://www.w3.org/2000/01/rdf-schema#>
	 * 
	 * @param resource
	 */
	public SemanticResource(Resource resource) {
		//setting the WebLab resource associated to this semantic resource
		this.resource = resource;
		
		//init loader & load Jena model for the WebLab resource
		long start = System.currentTimeMillis();
		loader = new FastResourceToJenaModel(resource);
		System.out.println("creating loader for "+resource.getUri()+" in "+(System.currentTimeMillis() -start));
		start = System.currentTimeMillis();
		model = loader.bejh.getModel();
		System.out.println("creating model for "+resource.getUri()+" in "+(System.currentTimeMillis()- start));
		
		//init prefixes
		prefix = new HashSet<String>();
		prefix.add("PREFIX rdf:<http://www.w3.org/1999/02/22-rdf-syntax-ns#>");
		prefix.add("PREFIX rdfs:<http://www.w3.org/2000/01/rdf-schema#>");
	}
	
	/**
	 * To construct a semantic resource from a WebLab Resource.
	 * Use the given Jena Ontology Model Specification for the Jena model, and fill it with
	 * - Statements from the WebLab Resource.
	 * - Statements about the WebLab structure of the resource are also added in the model.
	 * - Classes, properties & instances from ontologies.
	 * 
	 * Set rdf and rdfs prefixes for SPARQL queries to :
	 * - PREFIX rdf:<http://www.w3.org/1999/02/22-rdf-syntax-ns#>
	 * - PREFIX rdfs:<http://www.w3.org/2000/01/rdf-schema#>
	 * 
	 * @param resource
	 * @param jenaOntModelSpec
	 * @param ontologiesURLs
	 */
	public SemanticResource(Resource resource, OntModelSpec jenaOntModelSpec, String... ontologiesURLs) {
		//setting the WebLab resource associated to this semantic resource
		this.resource = resource;
		
		//init loader & load Jena model for the WebLab resource
		loader = new FastResourceToJenaModel(resource);
		model = loader.bejh.getModel();
		
		//add ontologies to the Jena model
		model = ModelFactory.createOntologyModel(jenaOntModelSpec, model);
		for (String ontology: ontologiesURLs) {
			model.read(ontology);
		}
		
		//init prefixes
		prefix = new HashSet<String>();
		prefix.add("PREFIX rdf:<http://www.w3.org/1999/02/22-rdf-syntax-ns#>");
		prefix.add("PREFIX rdfs:<http://www.w3.org/2000/01/rdf-schema#>");
	}
	
	/**
	 * Add a prefix to prefixes added to SPARQL queries on this semantic resource
	 * @param prefix
	 */
	public void addPrefixForSPARQLQueries(String... prefix) {
		if (prefix != null && !prefix.equals("")) {
			//add prefix to the set
			for (String pref : prefix) {
				this.prefix.add(pref);
			}
		}
	}

	/**
	 * To change the default Jena Model on this semantic resource to an OntologyModel according
	 * the given Jena Ontology Model Specification and load given ontologies in it.
	 * The new Jena Model contains :
	 * - Statements of the existing model
	 * - Classes, properties & instances from ontologies.
	 * 
	 * @param resource
	 * @param jenaOntModelSpec
	 * @param ontologiesURLs
	 */
	public void loadAnInferWithOntologies(OntModelSpec jenaOntModelSpec, String... ontologiesURLs ) {

		//add ontologies to the Jena model
		model = ModelFactory.createOntologyModel(jenaOntModelSpec, model);
		for (String ontology: ontologiesURLs) {
			model.read(ontology);
		}
	}

	/**
	 * To execute a "select" SPARQL request on this semantic resource and get the response
	 * in a Jena ResultSet. RDF and RDFS prefixes have not to be defined in the SPARQL request because they are
	 * automatically added.
	 * 
	 * A select can be used, for example, to get all images contained in this resource with the given request :
	 * 
	 * SELECT ?image WHERE {
	 *		?image rdf:type <http://weblab-project.org/core/model/class/Image>
	 * }
	 *  
	 * In this case, resultSet will contains URIs of WebLab images of this semantic resource.
	 * @param SPARQLQuery
	 * @return the Jena ResultSet for this SPARQL query.
	 */
	public ResultSet selectAsJenaResultSet(String SPARQLQuery) {
		
		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(getPrefix()+SPARQLQuery);
		QueryExecution qexec = QueryExecutionFactory.create(query, model);
		try {
			//exec select
			return qexec.execSelect();
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			//close QueryExecution
			qexec.close();
		}
		return null;
	}
	
	/**
	 * To execute a "select" SPARQL request on this semantic resource and write the response
	 * in the output stream in the JSON form. RDF and RDFS prefixes have not to be defined
	 * in the SPARQL request because they are automatically added.
	 * 
	 * A select can be used, for example, to get all images contained in this resource with the given request :
	 * 
	 * SELECT ?image WHERE {
	 *		?image rdf:type <http://weblab-project.org/core/model/class/Image>
	 * }
	 *  
	 * In this case, the result to this query is wrote as JSON in the output stream like :
	 * 
	 * {
	 * "head": {
	 * 		"vars": [ "image" ]
	 * 		   } ,
	 * "results": {
	 * 		"bindings": [
	 * 			{
	 * 				"image": { "type": "uri" , "value": "weblab://anImage" }
	 * 			}
	 * 		]
	 *   }
     * }
	 * @param SPARQLQuery
	 * @param outStream
	 */
	public void selectAsJSON(String SPARQLQuery, OutputStream outStream) {
		
		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(getPrefix()+SPARQLQuery);
		QueryExecution qexec = QueryExecutionFactory.create(query, model);
		try {
			//exec select and format resultSet
			ResultSetFormatter.outputAsJSON(outStream, qexec.execSelect());
		} catch (ClosedException ce) {
			ce.printStackTrace();
		} finally {
			//close QueryExecution
			qexec.close();
		}
	}
	
	/**
	 * To execute a "select" SPARQL request on this semantic resource and write the response
	 * in the output stream in the RDF/XML form. RDF and RDFS prefixes have not to be defined
	 * in the SPARQL request because they are automatically added.
	 * 
	 * A select can be used, for example, to get all images contained in this resource with the given request :
	 * 
	 * SELECT ?image WHERE {
	 *		?image rdf:type <http://weblab-project.org/core/model/class/Image>
	 * }
	 *  
	 * In this case, the result to this query is wrote as RDF/XML in the output stream like :
	 * 
	 * <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
     * 		xmlns:rs="http://www.w3.org/2001/sw/DataAccess/tests/result-set#" >
     * 
     * 	 <rdf:Description rdf:nodeID="A0">
     * 		<rs:solution rdf:nodeID="A1"/>
     * 		<rs:resultVariable>image</rs:resultVariable>
     *      <rdf:type rdf:resource="http://www.w3.org/2001/sw/DataAccess/tests/result-set#ResultSet"/>
     *   </rdf:Description>
     *   <rdf:Description rdf:nodeID="A1">
     *   	<rs:binding rdf:nodeID="A2"/>
     *   </rdf:Description>
     *   <rdf:Description rdf:nodeID="A2">
     *   	<rs:value rdf:resource="weblab://anImage"/>
     *      <rs:variable>image</rs:variable>
     *   </rdf:Description>
     *   
     * </rdf:RDF>
     * 
	 * @param SPARQLQuery
	 * @param outStream
	 */
	public void selectAsRDF(String SPARQLQuery, OutputStream outStream) {

		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(getPrefix()+SPARQLQuery);
		QueryExecution qexec = QueryExecutionFactory.create(query, model);
		try {
			//exec select and format resultSet
			ResultSet res = qexec.execSelect();
			ResultSetFormatter.outputAsRDF(outStream, "RDF/XML" ,res);
		} catch (ClosedException ce) {
			ce.printStackTrace();
		} finally {
			//close QueryExecution
			qexec.close();
		}
	}
	
	/**
	 * To execute a "select" SPARQL request on this semantic resource and write the response
	 * in the output stream in the XML form. RDF and RDFS prefixes have not to be defined
	 * in the SPARQL request because they are automatically added.
	 * 
	 * A select can be used, for example, to get all images contained in this resource with the given request :
	 * 
	 * SELECT ?image WHERE {
	 *		?image rdf:type <http://weblab-project.org/core/model/class/Image>
	 * }
	 *  
	 * In this case, the result to this query is wrote as XML in the output stream like :
	 * 
	 *  <?xml version="1.0"?>
	 *	<sparql xmlns="http://www.w3.org/2005/sparql-results#">
	 *		<head>
	 *			<variable name="image"/>
	 *		</head>
	 *		<results>
	 *			<result>
	 *				<binding name="image">
	 *					<uri>weblab://anImage</uri>
	 *				</binding>
	 *			</result>
	 *		</results>
	 *	</sparql>
     * 
	 * @param SPARQLQuery
	 * @param outStream
	 */
	public void selectAsXML(String SPARQLQuery, OutputStream outStream) {

		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(getPrefix()+SPARQLQuery);
		QueryExecution qexec = QueryExecutionFactory.create(query, model);
		try {
			//exec select and format resultSet
			ResultSet res = qexec.execSelect();
			ResultSetFormatter.outputAsXML(outStream, res);
		} catch (ClosedException ce) {
			ce.printStackTrace();
		} finally {
			//close QueryExecution
			qexec.close();
		}
	}
	
	/**
	 * To execute a "select" SPARQL request on this semantic resource and write the response
	 * in the output stream in the CSV form. RDF and RDFS prefixes have not to be defined
	 * in the SPARQL request because they are automatically added.
	 * 
	 * A select can be used, for example, to get all images contained in this resource with the given request :
	 * 
	 * SELECT ?image WHERE {
	 *		?image rdf:type <http://weblab-project.org/core/model/class/Image>
	 * }
	 *  
	 * In this case, the result to this query is wrote as CSV in the output stream like :
	 * 
	 *  image
	 *	weblab://anImage
     * 
	 * @param SPARQLQuery
	 * @param outStream
	 */
	public void selectAsCSV(String SPARQLQuery, OutputStream outStream) {

		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(getPrefix()+SPARQLQuery);
		QueryExecution qexec = QueryExecutionFactory.create(query, model);
		try {
			//exec select and format resultSet
			ResultSet res = qexec.execSelect();
			ResultSetFormatter.outputAsCSV(outStream, res);
		} catch (ClosedException ce) {
			ce.printStackTrace();
		} finally {
			//close QueryExecution
			qexec.close();
		}
	}
	
	/**
	 * To execute a "select" SPARQL request on this semantic resource and get all solutions that 
	 * point to a WebLab resource as a WebLab resources list.
	 * RDF and RDFS prefixes have not to be defined in the SPARQL request because they are 
	 * automatically added.
	 * 
	 * A select can be used, for example, to get all images, as WebLab resources,
	 * contained in this resource with the given request :
	 * 
	 * SELECT ?image WHERE {
	 *		?image rdf:type <http://weblab-project.org/core/model/class/Image>
	 * }
	 *  
	 * In this case, return a list that contains WebLab images contained in the semantic resource.
     * Note that all variables where solutions does not point to a WebLab resource a ignored in response.
     * 
	 * @param SPARQLQuery
	 * @return the list of WebLab resources
	 */
	public List<Resource> selectAsWeblabResources(String SPARQLQuery) {

		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(getPrefix()+SPARQLQuery);
		QueryExecution qexec = QueryExecutionFactory.create(query, model);
		try {
			//exec select and create resource list
			return listResourcesFromResultSet(qexec.execSelect());
		} catch (ClosedException ce) {
			ce.printStackTrace();
		} finally {
			//close QueryExecution
			qexec.close();
		}
		return null;
	}
	
	/**
	 * To execute a "describe" SPARQL request on this semantic resource and get the description as
	 * a news semantic resource. The semantic resource returned is composed by a PieceOfKnowledge and
	 * a default Jena Model about data of the PieceOfKnowledge.
	 * RDF and RDFS prefixes have not to be defined in the SPARQL request because they are 
	 * automatically added.
	 * 
	 * A describe can be used, for example, to get the complete description about all images 
	 * contained in this resource with the given request :
	 * 
	 * DESCRIBE ?image WHERE {
	 *		?image rdf:type <http://weblab-project.org/core/model/class/Image>
	 * }
	 *  
	 * In this case, return a new semantic resource composed by a PieceOfKnowledge that describe all images
	 * contained in the semantic resource and also the corresponding Jena model.
     * 
	 * @param SPARQLQuery
	 * @return the new semantic resource
	 */
	public SemanticResource describeAsSemanticResource(String SPARQLQuery) {
		
		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(SPARQLQuery) ;
		QueryExecution qexec = QueryExecutionFactory.create(getPrefix()+query, model);
		
		//creating a new PieceOfKnowledge
		PieceOfKnowledge pok = ResourceFactory.createResource("POK", ""+System.nanoTime(), PieceOfKnowledge.class);
		
		//writing pok data
		StringWriter writer = new StringWriter();
		qexec.execDescribe().write(writer, "RDF/XML");
		pok.setData(writer.getBuffer().toString());
		
		//creating semantic resource from pok
		SemanticResource ret = new SemanticResource(pok);
		
		//close QueryExecution
		qexec.close();
		return ret;
	}
	
	/**
	 * To execute a "describe" SPARQL request on this semantic resource and write the description as
	 * RDF/XML in the output stream.
	 * RDF and RDFS prefixes have not to be defined in the SPARQL request because they are 
	 * automatically added.
	 * 
	 * A describe can be used, for example, to get the complete description about all images 
	 * contained in this resource with the given request :
	 * 
	 * DESCRIBE ?image WHERE {
	 *		?image rdf:type <http://weblab-project.org/core/model/class/Image>
	 * }
	 *  
	 * In this case, the result to this query is wrote as RDF/XML in the output stream like :
	 * <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
	 * 			xmlns:j.0="http://weblab-project.org/core/model/property/structure/"
	 * 			xmlns:j.1="http://weblab-project.org/core/model/class/"
	 * 			xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
	 * 			xmlns:j.2="http://weblab-project.org/core/model/1.0/property/" >
	 * 
	 * 	 <rdf:Description rdf:about="weblab://anImage">
	 * 		<j.0:inComposedUnit rdf:resource="weblab://theComposedUnitforThisImage"/>
	 * 		<j.0:hasAnnotation rdf:resource="weblab://anWebLabAnnotationOnThisImage"/>
	 * 		<j.2:hasNativeContent rdf:resource="weblab://nativeContentForThisImage"/>
	 * 		<j.2:isExposedAs>http://example/anImage.jpg</j.2:isExposedAs>
	 * 		<rdf:type rdf:resource="http://weblab-project.org/core/model/class/Image"/>
	 *   </rdf:Description>
	 * </rdf:RDF>
     * 
	 * @param SPARQLQuery
	 * @param outStream
	 */
	public void describeAsRDF(String SPARQLQuery, OutputStream outStream) {
		
		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(SPARQLQuery) ;
		QueryExecution qexec = QueryExecutionFactory.create(getPrefix()+query, model);
		
		//exec describe and format resultSet
		qexec.execDescribe().write(outStream, "RDF/XML");
		
		//close QueryExecution
		qexec.close();
	}
	
	/**
	 * To execute a "describe" SPARQL request on this semantic resource and get the response an
	 * a Jena model.
	 * RDF and RDFS prefixes have not to be defined in the SPARQL request because they are 
	 * automatically added.
	 * 
	 * A describe can be used, for example, to get the complete description about all images 
	 * contained in this resource with the given request :
	 * 
	 * DESCRIBE ?image WHERE {
	 *		?image rdf:type <http://weblab-project.org/core/model/class/Image>
	 * }
	 *  
	 * In this case, the result to this query is a Jena model on statements of solutions.
	 * @param SPARQLQuery
	 * @return the Jena model
	 */
	public Model describeAsJenaModel(String SPARQLQuery) {
		
		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(SPARQLQuery) ;
		QueryExecution qexec = QueryExecutionFactory.create(getPrefix()+query, model);
		
		try {
			//exec describe and return resultSet
			return qexec.execDescribe();
		} finally {
			
			//close QueryExecution
			qexec.close();
		}
		
	}
	
	/**
	 * To execute a "construct" SPARQL request on this semantic resource and get the description as
	 * a news semantic resource. The semantic resource returned is composed by a PieceOfKnowledge and
	 * a default Jena Model about data of the PieceOfKnowledge. Data of the PieceOfKnowledge contains 
	 * statements returned in solutions.
	 * RDF and RDFS prefixes have not to be defined in the SPARQL request because they are 
	 * automatically added.
	 * 
	 * A construct can be used, for example, to get "isExposedAs" property values about all images 
	 * contained in this resource with the given request :
	 * 
	 * CONSTRUCT {
	 * 		?image <http://weblab-project.org/core/model/1.0/property/isExposedAs> ?exposed
	 * } WHERE {
	 *		?image rdf:type <http://weblab-project.org/core/model/class/Image>.
	 *		?image <http://weblab-project.org/core/model/1.0/property/isExposedAs> ?exposed
	 * }
	 *  
	 * In this case, return a new semantic resource composed by a PieceOfKnowledge that contains statements
	 * of solutions and also the corresponding Jena model.
     * 
	 * @param SPARQLQuery
	 * @return the new semantic resource
	 */
	public SemanticResource constructAsSemanticResource(String SPARQLQuery) {
		
		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(SPARQLQuery) ;
		QueryExecution qexec = QueryExecutionFactory.create(getPrefix()+query, model);
		
		//creating a new PieceOfKnowledge
		PieceOfKnowledge pok = ResourceFactory.createResource("POK", ""+System.nanoTime(), PieceOfKnowledge.class);
		
		//writing pok data
		StringWriter writer = new StringWriter();
		qexec.execConstruct().write(writer, "RDF/XML");
		pok.setData(writer.getBuffer().toString());
		
		//close QueryExecution
		qexec.close();
		
		//creating semantic resource from pok
		SemanticResource ret = new SemanticResource(pok);
		
		return ret;
	}
	
	/**
	 * To execute a "construct" SPARQL request on this semantic resource and get the description as
	 * a news semantic resource. The semantic resource returned is composed by a PieceOfKnowledge and
	 * a default Jena Model about data of the PieceOfKnowledge. Data of the PieceOfKnowledge contains 
	 * statements returned in solutions.
	 * RDF and RDFS prefixes have not to be defined in the SPARQL request because they are 
	 * automatically added.
	 * 
	 * A construct can be used, for example, to get "isExposedAs" property values about all images 
	 * contained in this resource with the given request :
	 * 
	 * CONSTRUCT {
	 * 		?image <http://weblab-project.org/core/model/1.0/property/isExposedAs> ?exposed
	 * } WHERE {
	 *		?image rdf:type <http://weblab-project.org/core/model/class/Image>.
	 *		?image <http://weblab-project.org/core/model/1.0/property/isExposedAs> ?exposed
	 * }
	 *  
	 * In this case, the result to this query is wrote as RDF/XML in the output stream like :
	 * 
	 * <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
	 * 			xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
	 * 			xmlns:j.0="http://weblab-project.org/core/model/1.0/property/" >
	 * 
	 * 	 <rdf:Description rdf:about="weblab://anImage">
	 * 		<j.0:isExposedAs>http://example/anImage.jpg</j.2:isExposedAs>
	 *   </rdf:Description>
	 *   
	 * </rdf:RDF>
	 * 
	 * @param SPARQLQuery
	 * @param outStream
	 */
	public void constructAsRDF(String SPARQLQuery, OutputStream outStream) {

		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(getPrefix()+SPARQLQuery) ;
		QueryExecution qexec = QueryExecutionFactory.create(query, model);
		
		//exec construct and format resultSet
		qexec.execConstruct().write(outStream, "RDF/XML");
		
		//close QueryExecution
		qexec.close();
	}
	
	/**
	 * To execute a "construct" SPARQL request on this semantic resource and get the response as a
	 * jena model. The returned jena model contains statements returned in solutions.
	 * RDF and RDFS prefixes have not to be defined in the SPARQL request because they are 
	 * automatically added.
	 * 
	 * A construct can be used, for example, to get "isExposedAs" property values about all images 
	 * contained in this resource with the given request :
	 * 
	 * CONSTRUCT {
	 * 		?image <http://weblab-project.org/core/model/1.0/property/isExposedAs> ?exposed
	 * } WHERE {
	 *		?image rdf:type <http://weblab-project.org/core/model/class/Image>.
	 *		?image <http://weblab-project.org/core/model/1.0/property/isExposedAs> ?exposed
	 * }
	 *  
	 * In this case, return a Jena model that contains statements of solutions.
     * 
	 * @param SPARQLQuery
	 * @return the Jena model
	 */
	public Model constructAsJenaModel(String SPARQLQuery) {
		
		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(SPARQLQuery) ;
		QueryExecution qexec = QueryExecutionFactory.create(getPrefix()+query, model);
		
		try {
			
			//exec construct and return Jena model
			return qexec.execConstruct();
		} finally {
			
			//close QueryExecution
			qexec.close();
		}
	}
	
	/**
	 * To execute a "ask" SPARQL request on this semantic resource and get the response as a boolean.
	 * RDF and RDFS prefixes have not to be defined in the SPARQL request because they are 
	 * automatically added.
	 * 
	 * A ask can be used, for example, to know if a resource is WebLab image with the given request :
	 * 
	 * "ASK {<weblab://anImage rdf:type <http://weblab-project.org/core/model/class/Image>}"
	 *  
	 * In this case, return true if weblab://anImage is a WebLab image, false otherwise.  
     * 
	 * @param SPARQLQuery
	 * @return boolean value
	 */
	public boolean ask(String SPARQLQuery) {
	
		//prepare a Jena Query & Jena QueryExecution
		Query query = QueryFactory.create(getPrefix()+SPARQLQuery) ;
		QueryExecution qexec = QueryExecutionFactory.create(query, model);
		try {
			//exec ask and return boolean
			return qexec.execAsk();
		} finally {

			//close QueryExecution
			qexec.close();
		}
	}
	
	/**
	 * To retrieve WebLab resources describe in the Jena ResultSet as a list
	 * @param the Jena resultSet
	 * @return the list of WebLab resources.
	 */
	private List<Resource> listResourcesFromResultSet(ResultSet res) {
		//prepare the return list
		ArrayList<Resource> ret = new ArrayList<Resource>();
		
		//iteration in resultset
		while (res.hasNext()) {
			QuerySolution solution = res.next();
			Iterator<String> varIterator = solution.varNames();
			
			//iteration on variables
			while (varIterator.hasNext()) {
				String varName = varIterator.next();
				if (solution.get(varName).isURIResource()) {
					
					//variable value for this result is a resource
					String resURI = solution.getResource(varName).getURI();
					if (loader.isDefinedResource(resURI)) {
						
						//the resource corresponds to a WebLab resource
						Resource resWL = loader.resourcesMap.get(resURI);
						if (resWL != null) {
							
							//add the WebLab resource to the list
							ret.add(resWL);
						}
					}
				}
			}
		}
		//retrun the list
		return ret;
	}
	
	/**
	 * To get all added prefixes
	 * @return
	 */
	private String getPrefix() {
		String prefixes="";
		for (String pref: prefix) {
			//concat prefix
			prefixes+=pref;
		}
		return prefixes;
	}
}
