/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2011 CASSIDIAN
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */

package org.ow2.weblab.core.helper.impl;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import junit.framework.Assert;

import org.junit.Test;
import org.ow2.weblab.core.extended.exception.WebLabCheckedException;
import org.ow2.weblab.core.extended.factory.AnnotationFactory;
import org.ow2.weblab.core.extended.factory.ResourceFactory;
import org.ow2.weblab.core.extended.jaxb.WebLabMarshaller;
import org.ow2.weblab.core.extended.ontologies.RDF;
import org.ow2.weblab.core.extended.ontologies.RDFS;
import org.ow2.weblab.core.extended.util.PoKUtil;
import org.ow2.weblab.core.helper.PoKHelper;
import org.ow2.weblab.core.model.Annotation;
import org.ow2.weblab.core.model.Resource;
import org.ow2.weblab.core.model.StringQuery;

import com.hp.hpl.jena.vocabulary.OWL;

/**
 * Test the JenaPoKHelperExtended Class (but use the methods inherited from
 * JenaAnnotationHelper)
 * 
 * @author gdupont Copyright EADS 21 Jan 2008
 * @version 0.1
 */
public class JenaPoKHelperImplTest {


	private final String rdf = "<rdf:RDF" + "	    xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"" + "	    xmlns:j.0=\"http://test/\">"
			+ "	      <rdf:Description rdf:about=\"weblab://test/test.resource.kiwi\">" + "	        <j.0:testproperty>BUNK1</j.0:testproperty>"
			+ "	        <j.0:isA>BUNK!</j.0:isA>" + "	      </rdf:Description>" + "</rdf:RDF>";



	private final PoKHelper helper;


	private final Resource res;


	private final Annotation a;


	public JenaPoKHelperImplTest() {
		this.res = ResourceFactory.createResource("annotation.helper.testCase", "res1", StringQuery.class);
		this.a = AnnotationFactory.createAndLinkAnnotation(this.res);
		this.helper = new JenaPoKHelper(this.a);
	}


	@Test
	public void testCreateStatement() {
		Assert.assertNull(this.a.getData());
		this.helper.setAutoCommitMode(false);
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label");
		Assert.assertNull(this.a.getData());
		this.helper.commit();
		Assert.assertNotNull(this.a.getData());
	}


	@Test
	public void testCreateStatementOnLanguage() {
		Assert.assertNull(this.a.getData());
		this.helper.setAutoCommitMode(false);
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label", "EN");
		Assert.assertNull(this.a.getData());
		this.helper.commit();
		Assert.assertNotNull(this.a.getData());
	}


	@Test
	public void testCreateStatementOnURI() {
		Assert.assertNull(this.a.getData());
		this.helper.setAutoCommitMode(true);
		this.helper.createResStat(this.res.getUri(), OWL.equivalentClass.getURI(), this.res.getUri());
		Assert.assertNotNull(this.a.getData());
	}


	@Test
	public void testCreateStatementWithAutoSerialisation() {
		Assert.assertNull(this.a.getData());
		this.helper.setAutoCommitMode(true);
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label");
		Assert.assertNotNull(this.a.getData());
	}


	@Test
	public void testCreateStatementWithSerialisationAndCommit() {
		Assert.assertNull(this.a.getData());
		this.helper.setAutoCommitMode(false);
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label");
		Assert.assertNull(this.a.getData());
		this.helper.commit();
		Assert.assertNotNull(this.a.getData());
	}


	@Test
	public void testRemoveStatement() {
		Assert.assertNull(this.a.getData());
		this.helper.setAutoCommitMode(true);
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label");
		Assert.assertNotNull(this.a.getData());
		Assert.assertTrue(this.helper.hasLitStat(this.res.getUri(), RDFS.LABEL, "label"));
		this.helper.removeLitStat(this.res.getUri(), RDFS.LABEL, "label");
		Assert.assertTrue(!this.helper.hasLitStat(this.res.getUri(), RDFS.LABEL, "label"));
	}


	@Test
	public void testHasPredicatesOnSubject() {
		this.helper.setAutoCommitMode(true);
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label");
		this.helper.createResStat(this.res.getUri(), RDF.TYPE, OWL.Class.getURI());
		Assert.assertTrue(this.helper.hasLitStat(this.res.getUri(), RDFS.LABEL, "label"));
		Assert.assertTrue(this.helper.hasResStat(this.res.getUri(), RDF.TYPE, OWL.Class.getURI()));
	}


	public void testGetPredicatesOnSubject() {
		this.helper.setAutoCommitMode(true);
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label");

		Set<String> predicates = this.helper.getPredsOnSubj(this.res.getUri());
		Assert.assertEquals(predicates.size(), 1);
		Assert.assertEquals(predicates.iterator().next(), RDFS.LABEL);

		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label2");
		predicates = this.helper.getPredsOnSubj(this.res.getUri());
		Assert.assertEquals(predicates.size(), 1);
		Assert.assertEquals(predicates.iterator().next(), RDFS.LABEL);

		this.helper.createResStat(this.res.getUri(), RDF.TYPE, OWL.Class.getURI());
		predicates = this.helper.getPredsOnSubj(this.res.getUri());
		Assert.assertEquals(predicates.size(), 2);
		Assert.assertTrue(predicates.contains(RDF.TYPE));

	}


	@Test
	public void testGetPredicateValuesOnSubject() {
		this.helper.setAutoCommitMode(true);
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label");
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label2");
		Collection<String> values = this.helper.getLitsOnPredSubj(this.res.getUri(), RDFS.LABEL);
		Assert.assertEquals(2, values.size());
		Assert.assertTrue(values.contains("label"));
		Assert.assertTrue(values.contains("label2"));

		values = this.helper.getRessOnPredSubj(this.res.getUri(), RDFS.LABEL);
		Assert.assertEquals(0, values.size());

		this.helper.createResStat(this.res.getUri(), RDF.TYPE, OWL.Class.getURI());
		values = this.helper.getRessOnPredSubj(this.res.getUri(), RDF.TYPE);
		Assert.assertEquals(1, values.size());
		Assert.assertTrue(values.contains(OWL.Class.getURI()));

	}


	@Test
	public void testGetPredicateValuesOnSubjectOnLanguage() {
		this.helper.setAutoCommitMode(true);
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label", "EN");
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "libellé", "FR");
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label2", "EN");
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "libellé2", "FR");


		Collection<String> values = this.helper.getLitsOnPredSubj(this.res.getUri(), RDFS.LABEL, "EN");
		Assert.assertEquals(2, values.size());
		Assert.assertTrue(values.contains("label"));
		Assert.assertTrue(values.contains("label2"));

		final Collection<String> values2 = this.helper.getLitsOnPredSubj(this.res.getUri(), RDFS.LABEL, "FR");
		Assert.assertEquals(2, values2.size());
		Assert.assertTrue(values2.contains("libellé"));
		Assert.assertTrue(values2.contains("libellé2"));

		values = this.helper.getRessOnPredSubj(this.res.getUri(), RDFS.LABEL);
		Assert.assertEquals(0, values.size());

		this.helper.createResStat(this.res.getUri(), RDF.TYPE, OWL.Class.getURI());
		values = this.helper.getRessOnPredSubj(this.res.getUri(), RDF.TYPE);
		Assert.assertEquals(1, values.size());
		Assert.assertTrue(values.contains(OWL.Class.getURI()));

	}


	@Test
	public void testIsDefinedResource() {
		this.helper.setAutoCommitMode(true);
		this.helper.createLitStat(this.res.getUri(), RDFS.LABEL, "label");
		Assert.assertFalse(this.helper.isDefinedResource("label"));
		Assert.assertTrue(this.helper.isDefinedResource(this.res.getUri()));

		this.helper.createResStat(this.res.getUri(), RDF.TYPE, OWL.Class.getURI());
		Assert.assertTrue(this.helper.isDefinedResource(OWL.Class.getURI()));
	}


	@Test
	public void testLoadModel() {// XXX
		this.helper.setAutoCommitMode(true);
		try {
			PoKUtil.setPoKData(this.a, this.rdf);
		} catch (final WebLabCheckedException wlce) {
			wlce.printStackTrace();
			Assert.fail(wlce.getMessage());
		}
		this.helper.setPoK(this.a);
		final Set<String> predicates = this.helper.getPredsOnSubj("weblab://test/test.resource.kiwi");
		Assert.assertNotNull(predicates);
		Assert.assertEquals(predicates.size(), predicates.size());
		for (final String predicate : predicates) {
			final List<String> predicateValues = this.helper.getLitsOnPredSubj("weblab://test/test.resource.kiwi", predicate);
			Assert.assertNotNull(predicateValues);
			if (predicate.equals("http://test/testproperty")) {
				Assert.assertEquals(1, predicateValues.size());
				Assert.assertEquals("BUNK1", predicateValues.iterator().next());
			} else if (predicate.equals("http://test/isA")) {
				Assert.assertEquals(1, predicateValues.size());
				Assert.assertEquals("BUNK!", predicateValues.iterator().next());
			} else {
				Assert.fail();
			}
		}
	}



	@Test
	public void testNsPrefix() {
		this.helper.setAutoCommitMode(false);
		final int size = this.helper.getNSPrefixes().size();
		this.helper.setNSPrefix("weblab", "http://weblab.eads.com/property/");
		Assert.assertFalse(size == this.helper.getNSPrefixes().size());
		this.helper.createLitStat(this.res.getUri(), "http://weblab.eads.com/property/prop1", "obj1");
		this.helper.createLitStat(this.res.getUri(), "http://weblab.eads.com/property/prop2", "obj2");
		this.helper.setAutoCommitMode(true);
		this.helper.createLitStat(this.res.getUri(), "http://weblab.eads.com/property/prop3", "weblab://theUriOfAnotherRes/id1");
		final String theData = PoKUtil.getPoKData(this.helper.getPoK());
		Assert.assertFalse("weblab should have been used instead", theData.contains("j.0"));
	}



	@Test
	public void testWarnOnRemove() throws Exception {
		this.helper.setAutoCommitMode(false);
		this.helper.createLitStat(this.res.getUri(), "http://weblab.eads.com/property/prop1", "obj1");
		this.helper.createLitStat(this.res.getUri(), "http://weblab.eads.com/property/prop2", "obj2");
		this.helper.createLitStat(this.res.getUri(), "http://weblab.eads.com/property/prop3", "obj2");
		this.helper.createLitStat(this.res.getUri(), "http://weblab.eads.com/property/prop4", "obj2");
		this.helper.createLitStat(this.res.getUri(), "http://weblab.eads.com/property/prop5", "obj2");
		this.helper.createLitStat(this.res.getUri(), "http://weblab.eads.com/prop/prop2", "obj2");
		this.helper.commit();
		final WebLabMarshaller wlm = new WebLabMarshaller();
		final ByteArrayOutputStream baos = new ByteArrayOutputStream();
		wlm.marshalResource(this.helper.getPoK(), baos);
		final ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
		final Annotation annot = wlm.unmarshal(bais, Annotation.class);

		final PoKHelper h = new JenaPoKHelper(annot);
		h.removeStatsWithLit("obj1");
		h.removeStatsWithPred("http://weblab.eads.com/property/prop2");
	}


	@Test
	public void testProcessResourcesInThread() throws InterruptedException, ExecutionException {
		final int nbThread = 10;
		final List<Callable<Boolean>> process = new ArrayList<Callable<Boolean>>();

		for (int i = 0; i < nbThread; i++) {
			process.add(new PoKHelperThread());
		}

		final ExecutorService executor = Executors.newFixedThreadPool(nbThread);
		final List<Future<Boolean>> results = executor.invokeAll(process, 5, TimeUnit.MINUTES);
		for (final Future<Boolean> future : results) {
			Assert.assertTrue(future.isDone());
			Assert.assertFalse(future.isCancelled());
			Assert.assertTrue(future.get().booleanValue());
		}
	}


	// TODO remove files used for this test and generate resources on the fly
	@Test
	public void testProcessComplexResourcesInThread() throws InterruptedException, ExecutionException {
		final int nbThread = 10;
		final List<Callable<Boolean>> process = new ArrayList<Callable<Boolean>>();

		for (int i = 0; i < nbThread; i++) {
			process.add(new PoKHelperComplexThread(i, "src/test/resources/xmlResources"));
		}

		final ExecutorService executor = Executors.newFixedThreadPool(nbThread);
		final List<Future<Boolean>> results = executor.invokeAll(process, 5, TimeUnit.MINUTES);
		for (final Future<Boolean> future : results) {
			Assert.assertTrue(future.isDone());
			Assert.assertFalse(future.isCancelled());
			Assert.assertTrue(future.get().booleanValue());
		}
	}

}
