/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 *  
 * Copyright (C) 2004 - 2011 Cassidian, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */

package org.ow2.weblab.core;

import java.io.File;
import java.io.StringWriter;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Calendar;
import java.util.Date;

import org.junit.Assert;
import org.junit.Test;
import org.ow2.weblab.core.extended.exception.WebLabCheckedException;
import org.ow2.weblab.core.extended.factory.AnnotationFactory;
import org.ow2.weblab.core.extended.factory.MediaUnitFactory;
import org.ow2.weblab.core.extended.factory.ResourceFactory;
import org.ow2.weblab.core.extended.factory.SegmentFactory;
import org.ow2.weblab.core.extended.jaxb.WebLabMarshaller;
import org.ow2.weblab.core.extended.ontologies.DublinCore;
import org.ow2.weblab.core.extended.ontologies.WebLabProcessing;
import org.ow2.weblab.core.extended.util.PoKUtil;
import org.ow2.weblab.core.helper.PoKHelper;
import org.ow2.weblab.core.helper.impl.JenaPoKHelper;
import org.ow2.weblab.core.model.Annotation;
import org.ow2.weblab.core.model.Audio;
import org.ow2.weblab.core.model.Coordinate;
import org.ow2.weblab.core.model.Document;
import org.ow2.weblab.core.model.Image;
import org.ow2.weblab.core.model.LinearSegment;
import org.ow2.weblab.core.model.SpatialSegment;
import org.ow2.weblab.core.model.Text;
import org.ow2.weblab.core.model.Video;
import org.ow2.weblab.core.model.processing.WProcessingAnnotator;
import org.purl.dc.elements.DublinCoreAnnotator;

import eu.virtuoso.VirtuosoAnnotator;


/**
 * Unit tests that create document in java and serialize them in target folder.
 * It is used as a tutorial for the creation of Documents using "extented", "annotator" and "helper".
 */
public class DocumentCreation {


	/**
	 * Creation of a simple Document that only contains three Text MediaUnit.
	 */
	@Test
	public void createSimpleTextDocument() throws WebLabCheckedException {

		/*
		 * Document creation.
		 * Use of ResourceFactory in extended.jar Arguments are the id the resource creator, id of the resource given
		 * the creator's knowledge
		 * of the world, class (extending resource).
		 * The first two parameters are used to generate a unique URI (according to the creator's knowledge of the
		 * world...)
		 */
		final Document doc = ResourceFactory.createResource("test", "d-0", Document.class);


		/*
		 * Creation of a Text MediaUnit inside the Document. With a unique based on the one of the Document.
		 */
		final Text introduction = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);


		/*
		 * Add the text content inside the mediaUnit.
		 */
		introduction.setContent("This is the introduction");


		/*
		 * Another text
		 */
		final Text section = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		section.setContent("First section\nThis is a text...");


		/*
		 * Another text
		 */
		final Text conclusion = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		conclusion.setContent("Conclusion\nThis is a versy simple example.");

		/*
		 * Use a marshaler to serialise the XML file. (the same method exists for streams, writers or files).
		 */
		final WebLabMarshaller marshaler = new WebLabMarshaller();
		marshaler.marshalResource(doc, new File("target/doc_simple.xml"));
		marshaler.marshalResource(doc, System.out);
		marshaler.marshalResource(doc, new StringWriter());
	}



	/**
	 * Creation of the same document but including annotations.
	 * As defined in the model, Annotations have a field name data, containing RDF/XML. When using Java directly this
	 * field is a DOM Document. More over, every
	 * resources may contain from 0 to n annotations.
	 * WebLab provides various way to set this field without handling this complex object. Each one are presented in the
	 * following code.
	 */
	@Test
	public void createDocumentWithSimpleAnnotation() throws WebLabCheckedException {
		final Document doc = ResourceFactory.createResource("test", "d-1", Document.class);

		final Text introduction = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		introduction.setContent("This is the introduction");
		final Text section = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		section.setContent("First section\nThis is a text...");
		final Text conclusion = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		conclusion.setContent("Conclusion\nThis is a versy simple example.");

		/*
		 * The simplest one, using the annotator, a library introduced in WebLab 1.2
		 * Annotator are dedicated to a single ontology. By default you have DublinCore, WebLabProcessing,
		 * WebLabRetrieval, WebLabModel and DublinCoreTerms.
		 * An annotator take as parameter the Resource, in which it will be added. Every statement has, by default, the
		 * resource as subject.
		 * For instance,
		 */
		final DublinCoreAnnotator annotator = new DublinCoreAnnotator(doc);
		annotator.writeCreator("WebLab resource sample tests");
		annotator.writeSource("http://weblab-project.org/");

		final Calendar cal = Calendar.getInstance();
		cal.set(2010, 5, 27, 11, 48);
		annotator.writeDate(cal.getTime());

		/*
		 * You may also want to use more complex process.
		 * In this case, you can use the rdf-helper. But prior to do that, you need to create an annotation (using
		 * extended)
		 */
		final Annotation annot = AnnotationFactory.createAndLinkAnnotation(doc);

		/*
		 * You have mainly 4 types of helpers:
		 * PoKHelper and PoKHelperExtended that can read and write into a single annotation.
		 * ResourceHelper and ResourceHelperExtented that can only read, but into every annotation on a given resource
		 * (recursively).
		 */
		final PoKHelper helper = new JenaPoKHelper(annot);

		helper.createLitStat(doc.getUri(), DublinCore.TITLE_PROPERTY_NAME, "The title.");
		helper.createResStat(annot.getUri(), WebLabProcessing.IS_PRODUCED_BY,
				"http://websvn.ow2.org/filedetails.php?repname=weblab&path=/trunk/WebLabHelpers/samples/src/test/java/org/ow2/weblab/core/DocumentCreation.java");
		helper.createLitStat(annot.getUri(), "http://weblab.ow2.org/core/1.2/ontology/processing#isExposedAt",
				"http://weblab-project.org/index.php?title=WebLab&oldid=237");


		final Annotation annot2 = AnnotationFactory.createAndLinkAnnotation(doc);

		/*
		 * If you use you own libraries to handle RDF/XML, and that you API does not allow the export of a DOM Document
		 * you still have, the last method. By
		 * direcly adding the RDF/XML String.
		 */
		final String rdfXml = "<rdf:RDF xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">"
				+ "<rdf:Description rdf:about=\"" + doc.getUri() + "\"><dc:language>en</dc:language></rdf:Description></rdf:RDF>";
		PoKUtil.setPoKData(annot2, rdfXml);


		new WebLabMarshaller().marshalResource(doc, new File("target/doc_annot.xml"));

		/*
		 * You can also unmarshal a resource from an XML file.
		 */
		final Document doc2 = new WebLabMarshaller().unmarshal(new File("target/doc_annot.xml"), Document.class);
		new WebLabMarshaller().marshalResource(doc2, new File("target/doc_annot2.xml"));

		Assert.assertTrue(doc.getUri().equals(doc2.getUri()));
		Assert.assertTrue(doc.getAnnotation().size() == doc2.getAnnotation().size());
		Assert.assertTrue(doc.getMediaUnit().size() == doc2.getMediaUnit().size());
	}



	/**
	 * This is just a simple test using many text units in various languages.
	 */
	@Test
	public void createMultilingualDoc() throws WebLabCheckedException {
		final Document doc = ResourceFactory.createResource("language.detection.test", "simple-doc", Document.class);

		final Text enUnit = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		final String en = "I want to crash the system by adding a little English.";
		enUnit.setContent(en);

		final Text frUnit = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		final String fr = "Je voudrais faire planter le système en ajoutant un peu de français.";
		frUnit.setContent(fr);

		final Text deUnit = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		final String de = "Ich möchte das System zum Absturz, indem Sie ein wenig Deutsch.";
		deUnit.setContent(de);

		final Text ruUnit = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		final String ru = "Я хочу к краху системы, добавив немного русский.";
		ruUnit.setContent(ru);

		final Text ptUnit = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		final String pt = "Eu quero derrubar o sistema, adicionando um pouco de Português.";
		ptUnit.setContent(pt);

		final Text roUnit = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		final String ro = "Vreau să prăbuşirii sistemului prin adăugarea de un român putin.";
		roUnit.setContent(ro);

		final Text esUnit = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		final String es = "Quiero bloquear el sistema, añadiendo un poco de español.";
		esUnit.setContent(es);

		final Text bgUnit = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		final String bg = "Искам да катастрофа на системата чрез добавяне на малко български.";
		bgUnit.setContent(bg);

		final Text daUnit = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		final String da = "Jeg ønsker at gå ned i systemet ved at tilføje en smule dansk.";
		daUnit.setContent(da);

		final Text elUnit = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		final String el = "Θα συντριβή του συστήματος, προσθέτοντας λίγο ελληνικά.";
		elUnit.setContent(el);

		final Text isUnit = MediaUnitFactory.createAndLinkMediaUnit(doc, Text.class);
		final String is = "Ég vil að hrun í kerfinu með því að bæta smá íslensku.";
		isUnit.setContent(is);

		new WebLabMarshaller().marshalResource(doc, new File("target/doc_multi_lingual.xml"));
	}




	/**
	 * This is the code used to generate the complex in WebLab presentations.
	 */
	@Test
	public void createComplexMultiDoc() throws WebLabCheckedException, URISyntaxException {

		/*
		 * Document creation.
		 */
		final Document document = ResourceFactory.createResource("video", "res_584697", Document.class);

		/*
		 * Create an annotator dedicated to the annotation of document using properties defined by dublin core ontology.
		 */
		final DublinCoreAnnotator dcAnnot = new DublinCoreAnnotator(document);

		/*
		 * Add the source of the video
		 */
		dcAnnot.writeSource("http://aljazeera.net/portal");

		/*
		 * Add the mime type of the video file
		 */
		dcAnnot.writeFormat("video/mp4");

		/*
		 * Create another annotator. This one dedicated to the annotation of document using properties defined by WebLab
		 * Processing ontology.
		 */
		WProcessingAnnotator wpAnnot = new WProcessingAnnotator(document);


		/*
		 * Create a Java Date
		 */
		final Calendar cal = Calendar.getInstance();
		cal.set(2007, 3, 9, 17, 37, 53);
		final Date gatheringDate = cal.getTime();

		/*
		 * Add the gathering date (yes it was a long time ago)
		 */
		wpAnnot.writeGatheringDate(gatheringDate);

		wpAnnot.writeExposedAs("ftp://ftp.myHost/video/res_584697.mp4");

		/*
		 * Create a marshaler, that eases the use of JAXB marshalling process.
		 */
		final WebLabMarshaller wlm = new WebLabMarshaller();

		wlm.marshalResource(document, new File("target/res_584697-v1.xml"));


		/*
		 * Create a video unit inside the document
		 */
		final Video video = MediaUnitFactory.createAndLinkMediaUnit(document, Video.class);

		wpAnnot = new WProcessingAnnotator(video);
		wpAnnot.writeNormalisedContent(new URI("rtp://servX1/" + video.getUri().hashCode()));

		wlm.marshalResource(document, new File("target/res_584697-v2.xml"));



		/*
		 * Create 4 images inside the document. They are extracted from the video and have a normalised content URI
		 */
		final Image image1 = MediaUnitFactory.createAndLinkMediaUnit(document, Image.class);
		wpAnnot = new WProcessingAnnotator(image1);
		VirtuosoAnnotator vAnnot = new VirtuosoAnnotator(image1);
		vAnnot.writeExtractedFrom(new URI(video.getUri()));
		wpAnnot.writeNormalisedContent(new URI("http://10.134.0.1/" + image1.getUri().hashCode()));

		final Image image2 = MediaUnitFactory.createAndLinkMediaUnit(document, Image.class);
		wpAnnot = new WProcessingAnnotator(image2);
		vAnnot = new VirtuosoAnnotator(image2);
		vAnnot.writeExtractedFrom(new URI(video.getUri()));
		wpAnnot.writeNormalisedContent(new URI("http://10.134.0.1/" + image2.getUri().hashCode()));

		final Image image3 = MediaUnitFactory.createAndLinkMediaUnit(document, Image.class);
		wpAnnot = new WProcessingAnnotator(image3);
		vAnnot = new VirtuosoAnnotator(image3);
		vAnnot.writeExtractedFrom(new URI(video.getUri()));
		wpAnnot.writeNormalisedContent(new URI("http://10.134.0.1/" + image3.getUri().hashCode()));

		final Image image4 = MediaUnitFactory.createAndLinkMediaUnit(document, Image.class);
		wpAnnot = new WProcessingAnnotator(image4);
		vAnnot = new VirtuosoAnnotator(image4);
		vAnnot.writeExtractedFrom(new URI(video.getUri()));
		wpAnnot.writeNormalisedContent(new URI("http://10.134.0.1/" + image4.getUri().hashCode()));



		wlm.marshalResource(document, new File("target/res_584697-v3.xml"));



		/*
		 * Create an audio unit. It is extracted from the video and has a normalised content URI
		 */
		final Audio audio = MediaUnitFactory.createAndLinkMediaUnit(document, Audio.class);
		wpAnnot = new WProcessingAnnotator(audio);
		vAnnot = new VirtuosoAnnotator(audio);
		vAnnot.writeExtractedFrom(new URI(video.getUri()));
		wpAnnot.writeNormalisedContent(new URI("rtsp://servideo/" + audio.getUri().hashCode()));



		wlm.marshalResource(document, new File("target/res_584697-v4.xml"));



		/*
		 * Create the text content of the audio transcription (speech-to-text).
		 */
		final Text transcription = MediaUnitFactory.createAndLinkMediaUnit(document, Text.class);
		wpAnnot = new WProcessingAnnotator(transcription);
		vAnnot = new VirtuosoAnnotator(transcription);
		vAnnot.writeTranscriptionOf(new URI(audio.getUri()));

		transcription
				.setContent("قتل 25 شخصا وأصيب 35 آخرون في انفجار قوي في شمال غربي باكستان، كما قتل أربعة أشخاص وأصيب اثنان آخران في قصف صاروخي استهدف سيارة في إقليم شمال وزيرستان، مما دفع العشرات من المتضررين من أمثال هذا القصف إلى التظاهر أمام مقر البرلمان الباكستاني.");


		wlm.marshalResource(document, new File("target/res_584697-v5.xml"));


		/*
		 * Create a text section extracted from a segment in an image.
		 */
		final Text recognisedText = MediaUnitFactory.createAndLinkMediaUnit(document, Text.class);
		recognisedText.setContent("A Message from Shaykh Usama bin Ladin to the America People");
		wpAnnot = new WProcessingAnnotator(recognisedText);
		vAnnot = new VirtuosoAnnotator(recognisedText);

		/*
		 * Create the spatial segment in image 2 and set its position
		 */
		final SpatialSegment segment = SegmentFactory.createAndLinkSegment(image2, SpatialSegment.class);
		final Coordinate c1 = new Coordinate();
		c1.setX(85);
		c1.setY(282);
		segment.getCoordinate().add(c1);
		final Coordinate c2 = new Coordinate();
		c2.setX(453);
		c2.setY(282);
		segment.getCoordinate().add(c2);
		final Coordinate c3 = new Coordinate();
		c3.setX(453);
		c3.setY(311);
		segment.getCoordinate().add(c3);
		final Coordinate c4 = new Coordinate();
		c4.setX(85);
		c4.setY(311);
		segment.getCoordinate().add(c4);

		/*
		 * Link the recognised text with the spatial segment
		 */
		vAnnot.writeRecognisedFrom(new URI(image2.getUri()));


		wlm.marshalResource(document, new File("target/res_584697-v6.xml"));



		/*
		 * Extract named entities and create linearsegments in the english text
		 */
		final LinearSegment benLadenSegment = SegmentFactory.createAndLinkLinearSegment(recognisedText, 15, 37);
		wpAnnot.startInnerAnnotatorOn(new URI(benLadenSegment.getUri()));
		wpAnnot.writeRefersTo(new URI("http://knowledgeBase/Person/BenLaden#0"));
		wpAnnot.endInnerAnnotator();

		final LinearSegment americaSegment = SegmentFactory.createAndLinkLinearSegment(recognisedText, 45, 52);
		wpAnnot.startInnerAnnotatorOn(new URI(americaSegment.getUri()));
		wpAnnot.writeRefersTo(new URI("http://knowledgeBase/Location/America#5"));
		wpAnnot.endInnerAnnotator();


		wlm.marshalResource(document, new File("target/res_584697-v7.xml"));

		/*
		 * Going further...
		 * 
		 * We provide Javadocs and sources for every libraries. When using maven add -DdownloadJavadocs and
		 * -DdownloadSources or have a look at the Maven OW2 repository (http://maven.ow2.org/maven2/org/ow2/weblab/core/)
		 * 
		 * Check out the sources of WebLab services or portlets similar to your needs
		 * (svn://svn.forge.objectweb.org/svnroot/weblab/WebLabServices).
		 * - Information Extraction in text --> gate-extraction and simple-gazetteer
		 * - Index/Search in text --> solr-engine
		 * - Document storage --> file-repository or simple-file-repository
		 * - ... and many others
		 * Have a look at the Wiki: http://weblab-project.org/index.php?title=WebLab
		 * And we encourage you to use the forum in case of any question regarding the model, extended, annotator,
		 * helper... or whatever is related to the WebLab platform.
		 */
	}
}
